var Class = require("core").Class;
var Loader = require("core").Net.Loader;
var OpalRequest = require("core").Net.OpalRequest;
var OpalLoader = require("core").Net.OpalLoader;
var RequestResponseDecorator = require("./RequestResponseDecorator.js").RequestResponseDecorator;
var AbstractDecorator = require("core").Decorator.AbstractDecorator;
var Event = require('core').Event.Event;
var ErrorEvent = require('core').Event.ErrorEvent;


/**
 * @class CdfLayoutDecorator Responsible for loading <cdf:model target=".skeleton"> configuration. Make call thru opal to service specified in configuration 
 * @extends RequestResponseDecorator
 * @param {Object} configuration Configuration object. <br/><i>Example configuration object:</i>
 *                 <pre>
 *                  {
 *                      "opalLocation" : {
 *                          "url" :     "example.test.onetapi.pl",
 *                          "method" :  "exampleMethodGetLayout",
 *                          "params" :  {
 *                              "name1" :   "value",
 *                              "name2" :   "value"
 *                          }
 *                      } 
 *                  }
 *                 </pre><br/>
 *                 <dl>
 *                      <dt>url</dt><dd>Location of service - registered in OPAL</dd>
 *                      <dt>method</dt><dd>Name of method to be called</dd>
 *                      <dt>params</dt><dd>Specific method's params to be called with</dd>
 *                 </dl>
 * @param {Array} contentTypes Array of Strings representing various mime types like: <i>text/html</i>, <i>application/json</i> or patterns <i>application/*</i>
 */
var CdfLayoutDecorator = function(){
    /** @ignore */
    this.Extends = RequestResponseDecorator;
    /**
     * @private
     * @type {Request}
     */
    this._request = null; 
    /**
     * @private
     * @type {Response}
     */
    this._response = null; 
    /**
     * Sets request object for further processing - in response decoration.
     * @public
     * @param {Request} request Request object to process on.
     */
    this.decorateRequest = function(request){
        console.log("CdfLayoutDecorator/decorateRequest");
        this._request = request;
        this.dispatchEvent(new Event(AbstractDecorator.COMPLETE));
    };
    /**
     * Retrive layout configuration from service specified in configuration. Adds <cdf:model target=".skeleton"> tag to response body.
     * @public
     * @param {Response} request Response object to process on.
     */
    this.decorateResponse = function(response){
        this._response = response;
        if(this.configuration.hasOwnProperty("opalLocation")){
            try {
                var opalRequest = new OpalRequest(this.configuration.opalLocation);    
            }
            catch(err){
                var util = require('util');
                 console.log(err.message, err.stack, util.inspect(err, true, 20));
                 this.dispatchEvent(new Event(AbstractDecorator.ERROR, this.configuration));
                 return;
            }
            //TODO clone headers - and prepare for opal call
            //opalRequest.setHeaders(this._request.getHeaders());
            var opalLoader = new OpalLoader(opalRequest);   
            opalLoader.addEventListener(Loader.Event.LOADED, this._onLoad, this);
            opalLoader.addEventListener(Loader.Event.ERROR, this._onError, this);
            console.log("make opal call");
            opalLoader.load();
        }
    };
    /**
     * Handle loading success
     * @private
     */
    this._onLoad = function(e){
        console.log("opal call: onload", e.data);
        //TODO co z cache
        //opakowujemy zwrotke w znacznik cdf:model
        var layout = "<cdf:model target=\".skeleton\" content-type=\"application/json\">" + JSON.stringify(e.data.getBody().getResult()) + "</cdf:model>";
        var body = layout + this._response.getBody()
        this._response.setBody(body);
        this.dispatchEvent(new Event(AbstractDecorator.COMPLETE, this._response));
    };
    /**
     * Handle loading error
     * @private
     */
    this._onError = function(e){
        console.log("opal call: onerror", e.data);
        this.dispatchEvent(new Event(AbstractDecorator.ERROR, e.data));
    };

};
CdfLayoutDecorator = new Class( new CdfLayoutDecorator() );

exports.CdfLayoutDecorator = CdfLayoutDecorator;
