var Class = require("core").Class;
var AbstractDecorator = require("core").Decorator.AbstractDecorator;
var Event = require('core').Event.Event;
/**
 * @class RequestResponseDecorator Base class for decorators family used by ProxyLoader.
 * @extends AbstractDecorator
 * @param {Object} configuration Configuration object, specific for decorator
 * @param {Array} contentTypes Array of Strings representing various mime types like: <i>text/html</i>, <i>application/json</i> or patterns <i>application/*</i>
 */
var RequestResponseDecorator = function(){
    this.Extends = AbstractDecorator;
    this.configuration = null;
    this._contentTypes = null;
    this.initialize = function(configuration, contentTypes){
        
        this.configuration = (configuration)?configuration:{};

        if(contentTypes){
            this._contentTypes = contentTypes; 
        }
        else{
            this._contentTypes = ["text/html"];
        }
    };
    /**
     * Base request decoration method. Should be overloaded.
     * @public
     * @param {Request} request
     */
    this.decorateRequest = function(request){
        this.dispatchEvent(new Event(AbstractDecorator.COMPLETE, request));
    };
    /**
     * Dispatches overriden decorateResponse method. Checks if decorator can be used with current response content type.
     * @param {Response} response HTTP Response object
     * @final
     */
    this.dispatchDecorateResponse = function(response){        
        if(this._isSupportedContentType(response)){
            this.decorateResponse(response);   
        }
        else{
            this.dispatchEvent(new Event(AbstractDecorator.COMPLETE, response));
        }
    };
    /**
     * Base response decoration method. Should be overloaded.
     * @public
     * @param {Response} response
     */
    this.decorateResponse = function(response){
        this.dispatchEvent(new Event(AbstractDecorator.COMPLETE, response));
    };
    /**
     * Checks if response content time is supported by this decorator.
     * @private
     */
    this._isSupportedContentType = function(response){
        var i = 0;
        var count = this._contentTypes.length;
        while(i < count && !response.isContentType(this._contentTypes[i])){
            i++;
        }
        if(i < count){
            return true;
        }
        return false;
    }
};
RequestResponseDecorator = new Class( new RequestResponseDecorator() );

/**
 * @static
 * @public
 * @param {String} decoratorClassName Name or relative path to Decorator's class. Default path is (...)/proxyEngine/lib/decorators/
 * @param {Array} contentTypes Array of strings representing mime types. {@see #RequestResponseDecorator}
 * @param {Object} configuration Configuration object
 */
RequestResponseDecorator.create = function(decoratorClassName, contentTypes, configuration){
    var DecoratorClass = require("./" + decoratorClassName + ".js")[decoratorClassName];
    var decorator = new DecoratorClass(configuration, contentTypes);
    return decorator;  
};

exports.RequestResponseDecorator = RequestResponseDecorator;
