var Class = require("core").Class;
var RequestResponseDecorator = require("./RequestResponseDecorator.js").RequestResponseDecorator;
var AbstractDecorator = require("core").Decorator.AbstractDecorator;
var Event = require('core').Event.Event;
var ErrorEvent = require('core').Event.ErrorEvent;
var Request = require('core').Net.Request;
var Response = require('core').Net.Response;

/**
 * @class RevProxyDecorator Klasa odpowiada za zachowanie zgodności z starym mechanizmem revproxy. Podmienia znaczniki komentarza html na znaczniki cdf:model.
 * @extends RequestResponseDecorator
 * @requires AbstractDecorator
 * @requires Event
 * @requires ErrorEvent
 */
var RevProxyDecorator = function(){
    this.Extends = RequestResponseDecorator;


    /**
     *
     *
     */
    this.decorateResponse = function(response){

        // if response is really Response Object
        if(response instanceof Response){
            var body = response.getBody();

            try{
                if(this.configuration
                    && this.configuration.hasOwnProperty('section')
                    && this.configuration.section.hasOwnProperty('length')
                    && this.configuration.section.length>0){

                    for(var i=0,max=this.configuration.section.length;i<max;i++){
                        var section = this.configuration.section[i];
                        if(section.hasOwnProperty('name')){
                            var chunk = this._getBodyChunk(section, body);
                            body = body.replace(this._getPattern(section), chunk);
                        } else {
                            throw RevProxyDecorator.Exception.INVALID_SECTION_NAME;
                        }
                    }
                    response.setBody(body);
                }
                this.dispatchEvent(new Event(AbstractDecorator.COMPLETE, null));


            } catch(e){
                this.dispatchEvent(new ErrorEvent(AbstractDecorator.ERROR, e));
            }
        } else {
            this.dispatchEvent(new ErrorEvent(AbstractDecorator.ERROR, null, -3200, "RevProxyDecorator: Give me Response Object please!"));
        }
    };


    /**
     *
     *
     */
    this._getBodyChunk = function(section, body){

        // get pattern for section
        var pattern = this._getPattern(section);

        // if there is no matching pattern throw an error
        if(pattern===null){
            throw RevProxyDecorator.Exception.INVALID_SECTION_PATTERN(section.name);
        }

        // initialize chunk to be empty string
        var bodyChunk = "";

        // retrive chunk using pattern
        var matches = body.match(this._getPattern(section));
        if(matches && matches.length>1){
            bodyChunk = matches[1].trim();
        } else {
            bodyChunk = "";
        }
        
        // if chunk is empty and there is something to subsitite
        if(bodyChunk==="" && section.hasOwnProperty('subst')){
            bodyChunk = section.subst;
        }

        // if there is prepend then prepend string
        if(section.hasOwnProperty('prepend')){
            bodyChunk = section.prepend + bodyChunk;
        }

        // if there is append then append string
        if(section.hasOwnProperty('append')){
            bodyChunk = bodyChunk + section.append;
        }

        // cover string with CDF:MODEL tag
        bodyChunk = '<cfd:model content-type="text/html" target="' + section.name + '">\r\n'
                        + bodyChunk
                    + '\r\n</cdf:model>';
       
       return bodyChunk;
       
    };

    /**
     *
     *
     */
    this._getPattern = function(section){
        var patterns = {
            'document:title': /(<title(?:>|\s+[^>]*>)(.*?)<\/title>)/i,
            'document:head' : /(<head(?:>|\s+[^>]*>)(.*?)<\/head>)/i,
            'document:body' : /(<body(?:>|\s+[^>]*>)(.*?)(<\/body>|$))/i
        }

        if(section && section.hasOwnProperty('source')){
            if(patterns.hasOwnProperty(section.source)){
                return patterns[section.source]
            } else {
                return null;
            }
        } else if(section && section.hasOwnProperty('begin') && section.hasOwnProperty('end')){
            return new RegExp("<!--\\s*" + section.begin + "\\s*-->((.|\\s)*?)<!--\\s*" + section.end + "\\s*-->", 'im');
        } else {
            return null;
        }

    };
   
};

RevProxyDecorator = new Class( new RevProxyDecorator() );



/**
 * Namespace for exeptions messages.
 * @static
 * @constant
 * @namespace
 */
RevProxyDecorator.Exception = {};
/**
 * @static
 * @constant
 */
RevProxyDecorator.Exception.INVALID_SECTION_NAME = "There is no 'name' field which is required!";
/**
 * @static
 * @constant
 */
RevProxyDecorator.Exception.INVALID_SECTION_PATTERN = function(sectionName){
    return "There is no pattern for section named: '" + sectionName + "' in RevProxyDecorator class.";
}


exports.RevProxyDecorator = RevProxyDecorator;
